/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>. 
 */
package data;

import java.io.Serializable;

/**
 * A representation of a concept term, i.e. a term that is extracted
 * as relevant for a certain concept. Apart from the string representation
 * of its term, a weight value is present, denoting the relevance degree
 * of this term with the given concept.
 * 
 * @author Pantelis Ieronimakis
 * @author Dorothea Tsatsou
 */


import java.util.ArrayList;
import java.util.HashMap;

/**
 * The Class GAINEntities. Data structure that represents  
 * input from the GAIN user tracking 
 * and preference learning web service.
 * 
 * @author Pantelis Ieronimakis
 */
public class GAINEntities implements Serializable {
	
	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 7593397476677407453L;
	
	/** The object id field. */
	private String objectID;
	
	/** The parent object id field. */
	private String parentObjectId;
	
	/** The attributes field. */
	private Attributes attributes;
	
	/** The account id  field. */
	private String accountID;
	
	/** The _id field. */
	private String _id;
	
	/** The __v  field. */
	private double __v;
	
	/** The entities field. */
	private ArrayList<Entities> entities;
	
	/** The rating field. */
	private Double rating;
	
	/** The group field. */
	private Double group;
	
	/** The type field. */
	private Double type;

	/**
	 * Instantiates a new GAIN entities structure.
	 */
	public GAINEntities() {
		this.objectID = "";
		this.parentObjectId = "";
		this.attributes = new Attributes();
		this.accountID = "";
		this._id = "";
		this.__v = 0;
		this.entities = new ArrayList<Entities>();
		this.rating = 0.0;
		this.type = 0.0;
		this.group = 0.0;
		
	}

	/**
	 * Instantiates a new GAIN entities structure.
	 *
	 * @param oid the object id
	 * @param poid the parent object id
	 * @param atts the attributes
	 * @param aid the account id
	 * @param id the id field
	 * @param v the v field
	 * @param ents the entities
	 * @param rt the ratings
	 * @param gr the group
	 * @param tp the type
	 */
	public GAINEntities(String oid, String poid, Attributes atts, String aid,
			String id, double v, ArrayList<Entities> ents,Double rt,Double gr,Double tp) {
		this.objectID = oid;
		this.parentObjectId = poid;
		this.attributes = atts;
		this.accountID = aid;
		this._id = id;
		this.__v = v;
		this.entities = ents;
		this.rating = rt;
		this.group=gr;
		this.type=tp;
	}

	/**
	 * Gets the object id.
	 *
	 * @return the object id
	 */
	public String getObjectID() {
		return this.objectID;
	}

	/**
	 * Sets the object id.
	 *
	 * @param oid the new object id
	 */
	public void setObjectID(String oid) {
		this.objectID = oid;
	}

	/**
	 * Gets the parent object id.
	 *
	 * @return the parent object id
	 */
	public String getParentObjectId() {
		return this.parentObjectId;
	}

	/**
	 * Sets the parent object id.
	 *
	 * @param poid the new parent object id
	 */
	public void setParentObjectId(String poid) {
		this.parentObjectId = poid;
	}

	/**
	 * Gets the attributes.
	 *
	 * @return the attributes
	 */
	public Attributes getAttributes() {
		return this.attributes;
	}

	/**
	 * Sets the attributes.
	 *
	 * @param atts the new attributes
	 */
	public void setAttributes(Attributes atts) {
		this.attributes.setAttributeStart(atts.getAttributeStart());
		this.attributes.setAttributeEnd(atts.getAttributeEnd());
	}

	/**
	 * Gets the account id.
	 *
	 * @return the account id
	 */
	public String getAccountID() {
		return this.accountID;
	}

	/**
	 * Sets the account id.
	 *
	 * @param aid the new account id
	 */
	public void setAccountID(String aid) {
		this.accountID = aid;
	}

	/**
	 * Gets the id.
	 *
	 * @return the id
	 */
	public String getID() {
		return this._id;
	}

	/**
	 * Sets the id.
	 *
	 * @param id the new id
	 */
	public void setID(String id) {
		this._id = id;
	}

	/**
	 * Gets the v field.
	 * 
	 * @return the v field
	 */
	public double getV() {
		return this.__v;
	}

	/**
	 * Sets the v field.
	 *
	 * @param v the new v field
	 */
	public void setV(double v) {
		this.__v = v;
	}
	
	/**
	 * Sets the rating.
	 *
	 * @param r the new rating
	 */
	public void setRating(double r) {
		this.rating = r;
	}
	
	/**
	 * Gets the rating.
	 *
	 * @return the rating
	 */
	public double getRating() {
		return this.rating;
	}
	
	/**
	 * Sets the group.
	 *
	 * @param g the new group
	 */
	public void setGroup(double g) {
		this.group = g;
	}
	
	/**
	 * Gets the group.
	 *
	 * @return the group
	 */
	public double getGroup() {
		return this.group;
	}
	
	/**
	 * Sets the type.
	 *
	 * @param t the new type
	 */
	public void setType(double t) {
		this.type = t;
	}
	
	/**
	 * Gets the type.
	 *
	 * @return the type
	 */
	public double getType() {
		return this.type;
	}

	/**
	 * Gets the entities.
	 *
	 * @return the entities
	 */
	public ArrayList<Entities> getEntities() {
		return this.entities;
	}

	/**
	 * Sets the entities.
	 *
	 * @param ents the new entities
	 */
	public void setEntities(ArrayList<Entities> ents) {
		this.entities.clear();
		this.entities.addAll(ents);
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		String s = this.objectID + ";" + this.parentObjectId + ";";
		s += this.attributes.getAttributeStart() + ";"
				+ this.attributes.getAttributeEnd() + ";";
		s += this.accountID + ";" + this._id + ";" + this.__v + ";";

		for (int i = 0; i < this.entities.size(); i++) {
			String c = this.entities.get(i).getEntityTypeLabel() + ";"
					+ this.entities.get(i).getEntitySource() + ";"
					+ this.entities.get(i).getEntityLod() + ";"
					+ this.entities.get(i).getEntityType() + ";"
					+ this.entities.get(i).getEntityLabel() + ";"
					+ this.entities.get(i).getEntityConfidence() + ";"
					+ this.entities.get(i).getEntityRelevance() + ";";
			s += c + ";";
		}
		// return s.substring(0, s.lastIndexOf(";") - 1);
		return s.substring(0, s.lastIndexOf(";"));
	}

	/**
	 * The Class Entities.
	 */
	public class Entities {
		
		/** The type label. */
		private String typeLabel;
		
		/** The entity type. */
		private String entityType;
		
		/** The i source. */
		private String iSource;
		
		/** The i lod. */
		private String iLod;
		
		/** The i type. */
		private String iType;
		
		/** The i label. */
		private String iLabel;
		
		/** The i confidence. */
		private double iConfidence;
		
		/** The i relevance. */
		private double iRelevance;
		
		/** The concepts. */
		private HashMap<String,Double> concepts;

		/**
		 * Instantiates a new entities.
		 */
		public Entities() {
			super();
			this.typeLabel = "";
			this.entityType = "";
			this.iSource = "";
			this.iLod = "";
			this.iType = "";
			this.iLabel = "";
			this.iConfidence = 0;
			this.iRelevance = 0;
		}
		
		/* 
		 * Latest GAINEntities input constructor		 
		 */
		
		/**
		 * Instantiates a new entities.
		 *
		 * @param is the is
		 * @param ilod the ilod
		 * @param itype the itype
		 * @param ilabel the ilabel
		 * @param tplabel the tplabel
		 * @param entityType the entity type
		 * @param ic the ic
		 * @param ir the ir
		 * @param hs the hs
		 */
		public Entities(String is,String ilod, String itype, String ilabel, String tplabel,
				String entityType, double ic, double ir,HashMap<String,Double> hs) {
			super();
			this.iSource = is;
			this.iLod = ilod;
			this.iType = itype;
			this.iLabel = ilabel;
			this.typeLabel = tplabel;			
			this.entityType = entityType;			
			this.iConfidence = ic;
			this.iRelevance = ir;
			this.concepts = hs;
		}
		
		/**
		 * Instantiates a new entities.
		 *
		 * @param is the is
		 * @param ilod the ilod
		 * @param itype the itype
		 * @param ilabel the ilabel
		 * @param tplabel the tplabel
		 * @param entityType the entity type
		 * @param ic the ic
		 * @param ir the ir
		 */
		public Entities(String is,String ilod, String itype, String ilabel, String tplabel,
				String entityType, double ic, double ir) {
			super();
			this.iSource = is;
			this.iLod = ilod;
			this.iType = itype;
			this.iLabel = ilabel;
			this.typeLabel = tplabel;			
			this.entityType = entityType;			
			this.iConfidence = ic;
			this.iRelevance = ir;
			
		}
		
		/**
		 * Gets the newconcs.
		 *
		 * @return the newconcs
		 */
		public HashMap<String,Double> getNewconcs(){
			return this.concepts;
		}

		/**
		 * Gets the entity type label.
		 *
		 * @return the entity type label
		 */
		public String getEntityTypeLabel() {
			return this.typeLabel;
		}

		/**
		 * Sets the entity type label.
		 *
		 * @param intype the new entity type label
		 */
		public void setEntityTypeLabel(String intype) {
			this.typeLabel = intype;
		}
		
		
		/**
		 * Gets the entity type.
		 *
		 * @return the entity type
		 */
		public String getEntityType() {
			return this.entityType;
		}

		/**
		 * Sets the entity type.
		 *
		 * @param intype the new entity type
		 */
		public void setEntityType(String intype) {
			this.entityType = intype ;
		}

		/**
		 * Gets the entity source.
		 *
		 * @return the entity source
		 */
		public String getEntitySource() {
			return this.iSource;
		}

		/**
		 * Sets the entity source.
		 *
		 * @param is the new entity source
		 */
		public void setEntitySource(String is) {
			this.iSource = is;
		}

		/**
		 * Gets the entity lod.
		 *
		 * @return the entity lod
		 */
		public String getEntityLod() {
			return this.iLod;
		}

		/**
		 * Sets the entity lod.
		 *
		 * @param ilod the new entity lod
		 */
		public void setEntityLod(String ilod) {
			this.iLod = ilod;
		}

		/**
		 * Gets the i type.
		 *
		 * @return the i type
		 */
		public String getIType() {
			return this.iType;
		}

		/**
		 * Sets the i type.
		 *
		 * @param itype the new i type
		 */
		public void setIType(String itype) {
			this.iType = itype;
		}

		/**
		 * Gets the entity label.
		 *
		 * @return the entity label
		 */
		public String getEntityLabel() {
			return this.iLabel;
		}

		/**
		 * Sets the entity label.
		 *
		 * @param ilabel the new entity label
		 */
		public void setEntityLabel(String ilabel) {
			this.iLabel = ilabel;
		}

		/**
		 * Gets the entity confidence.
		 *
		 * @return the entity confidence
		 */
		public double getEntityConfidence() {
			return this.iConfidence;
		}

		/**
		 * Sets the entity confidence.
		 *
		 * @param ic the new entity confidence
		 */
		public void setEntityConfidence(double ic) {
			this.iConfidence = ic;
		}

		/**
		 * Gets the entity relevance.
		 *
		 * @return the entity relevance
		 */
		public double getEntityRelevance() {
			return this.iRelevance;
		}

		/**
		 * Sets the entity relevance.
		 *
		 * @param ir the new entity relevance
		 */
		public void setEntityRelevance(double ir) {
			this.iRelevance = ir;
		}

	}

	/**
	 * The Class Attributes.
	 */
	public class Attributes {
		
		/** The i start. */
		private double iStart;
		
		/** The i end. */
		private double iEnd;

		/**
		 * Instantiates a new attributes.
		 */
		public Attributes() {
			super();
			this.iStart = 0;
			this.iEnd = 0;
		}

		/**
		 * Instantiates a new attributes.
		 *
		 * @param is the is
		 * @param ie the ie
		 */
		public Attributes(double is, double ie) {
			super();
			this.iStart = is;
			this.iEnd = ie;
		}

		/**
		 * Gets the attribute start.
		 *
		 * @return the attribute start
		 */
		public double getAttributeStart() {
			return this.iStart;
		}

		/**
		 * Sets the attribute start.
		 *
		 * @param is the new attribute start
		 */
		public void setAttributeStart(double is) {
			this.iStart = is;
		}

		/**
		 * Gets the attribute end.
		 *
		 * @return the attribute end
		 */
		public double getAttributeEnd() {
			return this.iEnd;
		}

		/**
		 * Sets the attribute end.
		 *
		 * @param ie the new attribute end
		 */
		public void setAttributeEnd(double ie) {
			this.iEnd = ie;
		}
	}
	// @Override
	// public boolean equals(Object obj) {
	// if (!(obj instanceof GAINFullExport)) return false;
	// GAINFullExport otherEntry = (GAINFullExport)obj;
	// if (lemma.equals(otherLemma.getLemma()) &&
	// type.equals(otherLemma.getType()))
	// return true;
	// return false;
	// }

}
