/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>.
 */
package data;

/**
 * The Class Rule. Data structure that represents  
 * rules, as are received from the GAIN user tracking 
 * and preference learning web service.
 * 
 * @author Pantelis Ieronimakis
 */
public class Rule {

	/** The rule. */
	private String rule;
	
	/** The antecedent. */
	private String antecedent;
	
	/** The contecedent. */
	private String contecedent;
	
	/** The confidence. */
	private Double confidence;
	
	/** The support. */
	private Double support;

	/**
	 * Instantiates a new rule.
	 */
	public Rule() {
	}

	/**
	 * Instantiates a new rule.
	 *
	 * @param rule the rule
	 * @param antecedent the antecedent
	 * @param contecedent the contecedent
	 * @param confidence the confidence
	 * @param support the support
	 */
	public Rule(String rule, String antecedent, String contecedent,
			Double confidence, Double support) {
		this.rule = rule;
		this.antecedent = antecedent;
		this.contecedent = contecedent;
		this.confidence = confidence;
		this.support = support;

	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return ("Rule : " + this.rule + "\n" + "Antecedent : "
				+ this.antecedent + "\n" + "Contecedent : " + this.contecedent
				+ "\n" + "Confidence : " + this.confidence + "\n"
				+ "Support : " + this.support + "\n" + "\n");
	}

	/**
	 * The Class ConceptValue.
	 */
	public class ConceptValue {
		
		/** The concept. */
		private String concept;
		
		/** The value. */
		private String value;

		/**
		 * Instantiates a new concept value.
		 *
		 * @param concept the concept
		 * @param value the value
		 */
		public ConceptValue(String concept, String value) {
			this.concept = concept;
			this.value = value;
		}

		/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		public String toString() {
			return ("Concept : " + this.concept + "\t" + "Value : "
					+ this.value + "\n");
		}

	}
}
