/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>.
 */
package input;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.osgi.internal.signedcontent.Base64;

import data.GAINEntities;

import com.google.gson.Gson;
import com.google.gson.internal.LinkedHashTreeMap;
import com.google.gson.reflect.TypeToken;

/**
 * The Class readGainEntities. Reads input from the GAIN user tracking 
 * and preference learning web service.
 * 
 * @author Pantelis Ieronimakis
 */
public class readGainEntities {

	/**
	 * Gets the gain entities.
	 *
	 * @param inputUrl the input url
	 * @return the gain entities
	 */
	protected static List<LinkedHashTreeMap> getGainEntities(String inputUrl) {

		URL url = null;
		try {
			url = new URL(inputUrl);
			// "http://dev.wa.vse.cz/api/UA-XXXXXXX-Y/1/export/interests");
		} catch (MalformedURLException e2) {
			e2.printStackTrace();
		}
		URLConnection yc3 = null;
		try {
			yc3 = url.openConnection();
		} catch (IOException e2) {
			e2.printStackTrace();
		}
		String userpass = "YOUTUBE-TEST" + ":" + "YOUTUBE-TEST";
		String basicAuth = "Basic "
				+ new String(new Base64().encode(userpass.getBytes()));
		yc3.setRequestProperty("Authorization", basicAuth);
		BufferedReader in = null;
		try {
			in = new BufferedReader(new InputStreamReader(yc3.getInputStream()));
		} catch (IOException e) {
			e.printStackTrace();
		}
		Gson gson = new Gson();
		List<LinkedHashTreeMap> items = null;
		try {
			items = gson.fromJson(in.readLine(),
					new TypeToken<List<LinkedHashTreeMap>>() {
					}.getType());
		} catch (IOException e1) {
			e1.printStackTrace();
		}

		List<LinkedHashTreeMap> sessions = new Vector<LinkedHashTreeMap>();
		for (int i = 0; i < items.size(); i++) {
			sessions.add(items.get(i));
		}
		// System.out.println("EEEEEEEEEEEE" + sessions.get(10));
		return sessions;
	}

	/**
	 * Read Gain Entities.
	 *
	 * @param inputUrl the input url
	 * @return the list
	 */
	public static List<GAINEntities> readGainentities(String inputUrl) {
		List<GAINEntities> exp = new ArrayList<GAINEntities>();
		List<LinkedHashTreeMap> sessions = getGainEntities(inputUrl);
		Iterator it = sessions.iterator();
		while (it.hasNext()) {

			LinkedHashTreeMap next = (LinkedHashTreeMap) it.next();
			GAINEntities input = new GAINEntities();
			input.setObjectID((String) next
					.get("objectId")
					.toString()
					.substring(
							next.get("objectId").toString().lastIndexOf("/") + 1));
			input.setParentObjectId((String) next.get("parentObjectId"));

			LinkedHashTreeMap<String, String> lhmt = (LinkedHashTreeMap<String, String>) next
					.get("attributes");

			data.GAINEntities.Attributes atts = input.new Attributes();
			atts.setAttributeStart(Double.parseDouble(String.valueOf(lhmt
					.get("start"))));
			atts.setAttributeEnd(Double.parseDouble(String.valueOf(lhmt
					.get("end"))));

			input.setAttributes(atts);
			input.setAccountID((String) next.get("accountId"));
			input.setID((String) next.get("_id"));
			input.setV(Double.parseDouble(String.valueOf(next.get("__v"))));
			// System.out.println("test" + next.get("entities"));
			ArrayList<LinkedHashTreeMap<String, String>> al = (ArrayList<LinkedHashTreeMap<String, String>>) next
					.get("entities");
			// System.out.println("TEST" + al);

			ArrayList<data.GAINEntities.Entities> ents = new ArrayList<data.GAINEntities.Entities>();

			for (int i = 0; i < al.size(); i++) {

				ents.add(input.new Entities(al.get(i).get("source"), al.get(i)
						.get("lod"), al.get(i).get("type"), al.get(i).get(
						"label"), al.get(i).get("typeLabel"), al.get(i).get(
						"entityType"), Double.parseDouble(String.valueOf(al
						.get(i).get("confidence"))), Double.parseDouble(String
						.valueOf(al.get(i).get("relevance")))));
			}
			input.setEntities(ents);
			input.setRating(Double.parseDouble(String.valueOf(next
					.get("rating"))));
			input.setGroup(Double.parseDouble(String.valueOf(next.get("group"))));
			input.setType(Double.parseDouble(String.valueOf(next.get("type"))));

			exp.add(input);
			// System.out.println(input);
			// System.out.println(input);

		}

		return exp;
	}

}
