/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>.
 */
package input;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Vector;

import org.eclipse.osgi.internal.signedcontent.Base64;

/**
 * The Class readLatestFullExport. Reads input from the GAIN user tracking 
 * and preference learning web service.
 * 
 * @author Pantelis Ieronimakis
 */
public class readLatestFullExport {

	/**
	 * Gets the GAIN Full export.
	 *
	 * @param inputUrl the input url
	 * @return the GAIN Full export
	 */
	protected static Vector<String> getGAINFUllExport(String inputUrl) {
		URL url = null;
		try {
			url = new URL(inputUrl);
			// "http://dev.wa.vse.cz/api/UA-XXXXXXX-Y/1/export/interests");
		} catch (MalformedURLException e2) {
			e2.printStackTrace();
		}
		URLConnection yc2 = null;
		try {
			yc2 = url.openConnection();
		} catch (IOException e2) {
			e2.printStackTrace();
		}
		String userpass = "YOUTUBE-TEST" + ":" + "YOUTUBE-TEST";
		@SuppressWarnings("static-access")
		String basicAuth = "Basic "
				+ new String(new Base64().encode(userpass.getBytes()));
		yc2.setRequestProperty("Authorization", basicAuth);
		BufferedReader in = null;
		try {
			in = new BufferedReader(new InputStreamReader(yc2.getInputStream()));
		} catch (IOException e) {
			e.printStackTrace();
		}
		String inputLine;
		Vector<String> sessions = new Vector<String>();
		try {
			while ((inputLine = in.readLine()) != null) {

				sessions.add(inputLine);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return sessions;

	}

	/**
	 * Read gain full export.
	 *
	 * @param url the url
	 * @return the vector
	 */
	public static Vector<data.GAINFullExport> readGAINFullExport(String url) {
		Vector<data.GAINFullExport> exp = new Vector<data.GAINFullExport>();
		Vector<String> rawExp = getGAINFUllExport(url);

		String[] sessions = rawExp.get(0).split("\\},\\{");
		// System.out.println(sessions.length);
		for (int i = 0; i < sessions.length; i++) {
			data.GAINFullExport input = new data.GAINFullExport();

			String[] sarray = sessions[i].split(",");
			for (int j = 1; j < sarray.length; j++) {
//				System.out.println(sessions.length);
				try {

					// SESSION ID
					if (sarray[j].split(":")[0].replaceAll("[^a-zA-Z]", "")
							.equalsIgnoreCase("sessionId")) {
						input.setSessionID(Long.parseLong(sarray[j].split(":")[1]
								.substring(1,
										sarray[j].split(":")[1].length() - 1)));
					} else if (sarray[j].split(":")[0].replaceAll("[^a-zA-Z]",
							"").equalsIgnoreCase("objectId")) {
//						System.out.println(sarray[j].split("\":\"")[1]
//								.substring(sarray[j].split("\":\"")[1].lastIndexOf("/")+1,
//										sarray[j].split("\":\"")[1].indexOf("?") - 1));
						input.setObjectID(sarray[j].split("\":\"")[1].substring(
								0, sarray[j].split("\":\"")[1].length() - 1));
					} else if (sarray[j].split(":")[0].replaceAll("[^a-zA-Z]",
							"").equalsIgnoreCase("parentObjectId")) {
						input.setSubobjectID(sarray[j].split(":")[1]);
					} else if (sarray[j].split(":")[0].replaceAll("[^a-zA-Z]",
							"").equalsIgnoreCase("userId")) {
						input.setUserId(sarray[j].split(":")[1].replaceAll(
								"[^A-Za-z0-9]", ""));
					} else if (sarray[j].split(":")[0].replaceAll("[^a-zA-Z]",
							"").equalsIgnoreCase("interest")) {
//						System.out.println("VRIKE INTEREST : "
//								+ sarray[j].split(":")[1].replaceAll("[^\\d.]",
//										""));
						input.setInterest(Double.parseDouble(sarray[j]
								.split(":")[1].replaceAll("[^\\d.]", "")));
					} else {
						String[] parts5 = sarray[j].split(":");

						if (Double.parseDouble(parts5[1].replaceAll("[^\\d.]",
								"")) == 0.00)
							continue;
						input.putConcept(parts5[0].substring(1,
								parts5[0].length()), Double
								.parseDouble(parts5[1]
										.replaceAll("[^\\d.]", "")));

					}
				} catch (Exception e) {

				}

			}
			exp.add(input);
		}

		return exp;
	}

}
