/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>.
 */
package input;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.xml.crypto.Data;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import data.GAINEntities;
import data.ProfileConcept;

import org.eclipse.osgi.internal.signedcontent.Base64;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import profile.PlainUserProfileCreator;
import store.PMMLParserHandler;
import sys.Config;

/**
 * The Class readPMML. Reads input rules from the GAIN user tracking 
 * and preference learning web service.
 * 
 * @author Pantelis Ieronimakis
 */
public class readPMML {

	/**
	 * Gets the timestamp.
	 *
	 * @return the timestamp
	 */
	public static long getTimestamp() {
		java.util.Date today = new java.util.Date();
		java.sql.Timestamp ts = new java.sql.Timestamp(today.getTime());
		long tsTime = ts.getTime();
		return tsTime;
	}

	/**
	 * Xml output.
	 *
	 * @param Object the object
	 * @return the list
	 * @throws IOException Signals that an I/O exception has occurred.
	 */
	public List<ProfileConcept> xmlOutput(PMMLParserHandler Object)
			throws IOException {

		FileWriter fstream = new FileWriter("C:\\Users\\pieronim\\test.txt",
				true);
		BufferedWriter out = new BufferedWriter(fstream);

		Integer freq;
		List<String> keep = new ArrayList<String>();
		List<ProfileConcept> xmlList = new ArrayList<ProfileConcept>();

		for (int i = 0; i < Object.getAntecedents().size(); i++) {
			freq = 1;
			for (int l = 0; l < keep.size(); l++) {
				if (Object.getAntecedents().get(i).toString().toLowerCase()
						.equals(keep.get(l))) {

					freq++;
				}
			}

			List<String> sum = new ArrayList<String>();
			long tsTime = getTimestamp();
			String[] parts = Object.getAntecedents().get(i).split("AND");
			String keeper1 = null;
			for (int j = 0; j < parts.length; j++) {
				freq = 1;
				if (parts[j].substring(parts[j].indexOf("("))
						.replaceAll("\\s+", "").length() > 3) {

					String temp1 = (parts[j]
							.substring(0, parts[j].indexOf("(")).trim());

					String temp2 = (parts[j].substring(
							parts[j].indexOf("(") + 1, parts[j].length() - 2)
							.trim());
					// keeper1 = "d_o_" + temp2 + ":" + "l_o_" + temp1;
					if (temp2.equalsIgnoreCase("no")) {
						keeper1 = "-" + temp1.substring(0, 1)
								+ temp1.substring(1);
					}
					if (!temp2.equalsIgnoreCase("yes")) {
						keeper1 = temp2.substring(0, 1) + temp2.substring(1)
								+ ":" + temp1.substring(0, 1)
								+ temp1.substring(1);
					} else {
						keeper1 = temp1.substring(0, 1) + temp1.substring(1);
					}

					sum.add(keeper1);
				}
			}
			String temp = sum.toString()
					.substring(1, sum.toString().length() - 1)
					.replaceAll(",", " AND ");
			out.append(freq
					+ "\t"
					+ tsTime
					+ "\t"
					+ Object.getConfidence().get(i)
					+ "\t"
					+ "\t"
					+ "\t"
					+ "positive"
					+ "\t"
					+ sum.toString().substring(1, sum.toString().length() - 1)
							.replaceAll(",", " AND ") + "\tnull" + "\n");
			xmlList.add(new ProfileConcept(temp, freq, tsTime, true, Object
					.getConfidence().get(i), null));
			keep.add(Object.getAntecedents().get(i).toString().toLowerCase());

		}
		out.close();
		return xmlList;
	}

	/**
	 * Read pmml.
	 *
	 * @param inputUrl the input url
	 * @throws TransformerException the transformer exception
	 * @throws ParserConfigurationException the parser configuration exception
	 * @throws IOException Signals that an I/O exception has occurred.
	 * @throws SAXException the SAX exception
	 */
	public void readPMML(String inputUrl) throws TransformerException,
			ParserConfigurationException, IOException, SAXException {
		// pmmlToFile();

		readPMML b = new readPMML();
		PlainUserProfileCreator ts = new PlainUserProfileCreator();
		PMMLParserHandler a = new PMMLParserHandler();
		XMLReader parser = XMLReaderFactory.createXMLReader();
		parser.setContentHandler(a);
		parser.parse("C:\\Users\\pieronim\\myOutput.xml");
	}

	/**
	 * The main method.
	 *
	 * @param args the arguments
	 * @throws IOException Signals that an I/O exception has occurred.
	 * @throws ParserConfigurationException the parser configuration exception
	 * @throws SAXException the SAX exception
	 * @throws TransformerException the transformer exception
	 */
	public static void main(String[] args) throws IOException,
			ParserConfigurationException, SAXException, TransformerException {
		pmmlToFile("YOUTUBE-TEST", "1");
	}

	/**
	 * Pmml to file.
	 *
	 * @param accountID the account id
	 * @param userID the user id
	 * @throws ParserConfigurationException the parser configuration exception
	 * @throws IOException Signals that an I/O exception has occurred.
	 * @throws SAXException the SAX exception
	 * @throws TransformerFactoryConfigurationError the transformer factory configuration error
	 * @throws TransformerConfigurationException the transformer configuration exception
	 * @throws TransformerException the transformer exception
	 */
	public static void pmmlToFile(String accountID, String userID)
			throws ParserConfigurationException, IOException, SAXException,
			TransformerFactoryConfigurationError,
			TransformerConfigurationException, TransformerException {
		URL url = null;
		try {
			url = new URL("http://dev.wa.vse.cz/pl/api/" + accountID
					+ "/rules?uid=" + userID);
			// "http://dev.wa.vse.cz/api/UA-XXXXXXX-Y/1/export/interests");
		} catch (MalformedURLException e2) {
			e2.printStackTrace();
		}
		URLConnection yc = null;
		try {
			yc = url.openConnection();
		} catch (IOException e2) {
			e2.printStackTrace();
		}
		String userpass = "YOUTUBE-TEST" + ":" + "YOUTUBE-TEST";
		String basicAuth = "Basic "
				+ new String(new Base64().encode(userpass.getBytes()));
		yc.setRequestProperty("Authorization", basicAuth);
		BufferedReader in = null;
		try {
			in = new BufferedReader(new InputStreamReader(yc.getInputStream()));
		} catch (IOException e) {
			e.printStackTrace();
		}
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			URLConnection conn = url.openConnection();
			String userPassword = "YOUTUBE-TEST" + ":" + "YOUTUBE-TEST";
			conn.setRequestProperty("Authorization", basicAuth);
			conn.connect();
			Document doc = builder.parse(conn.getInputStream());

			TransformerFactory tfactory = TransformerFactory.newInstance();
			Transformer xform = tfactory.newTransformer();

			File myOutput = new File("myOutput.xml");
			xform.transform(new DOMSource(doc), new StreamResult(myOutput));
		} catch (Exception e) {

		}

	}
}
