/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>.
 */
package onto;

import store.ProfileIO;
import lifr.logic.firstorder.KnowledgeBase;
import lifr.util.KRSSFuzzyParser;

/**
 * Class used to retrieve the ad and/or user profiles from disk. 
 * In each case, the input files
 * are converted from the KRSS format to logic rules in Protein syntax.
 * 
 * @author Dorothea Tsatsou
 */
public class ProfileRetriever {
	
	/**
	 * Retrieve a specific user profile (stored locally) and convert it
	 * (from the stored KRSS format) to a knowledge base of
	 * logic rules.
	 * 
	 * @param userID the ID of the user
	 * 
	 * @return the knowledge base derived from the user profile in logic rules
	 */
	public static final KnowledgeBase getUserProfile(String userID){
    	KnowledgeBase kb = new KnowledgeBase();
    	String tbox = ProfileIO.loadUserProfile(userID);
    	KRSSFuzzyParser parser = new KRSSFuzzyParser(tbox);
    	kb = parser.getKB();
    	return kb;
    }

}
