/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>.
 */
package profile;

import input.readPMML;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactoryConfigurationError;

import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import store.PMMLParserHandler;
import sys.Config;
import data.Concept;
import data.GAINEntities;
import data.GAINFullExport;
import data.ProfileConcept;

/**
 * Class used to create-update a user profile based upon
 * translateGAINFullExport,translateGAINEntities and PMML parsing.
 * 
 * @author Pantelis Ieronimakis
 */
public class PlainUserProfileCreator {

	/**
	 * Parse function that gets the associated rules from a PMML file..
	 * 
	 * @param Object
	 *            the object
	 * @return the associated rules
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public List<ProfileConcept> getAssociatedRules(PMMLParserHandler Object)
			throws IOException {

		Integer freq;
		Vector<String> keep = new Vector<String>();
		List<ProfileConcept> associatedRules = new ArrayList<ProfileConcept>();

		for (int i = 0; i < Object.getAntecedents().size(); i++) {
			if (Object.getAntecedents().get(i).toString().contains(" AND ")
					|| Object.getAntecedents().get(i).toString()
							.contains(" and ")) {
				freq = 1;
				ArrayList<String> concepts = new ArrayList<String>();
				// Vector<String> sum = new Vector<String>();
				long tsTime = util.TimestampReader.getTimestamp();
				String[] parts = Object.getAntecedents().get(i).split("AND");
				String keeper1 = null;
				for (int j = 0; j < parts.length; j++) {
					freq = 1;
					if (parts[j].substring(parts[j].indexOf("("))
							.replaceAll("\\s+", "").length() > 3) {
						String temp1 = (parts[j].substring(0,
								parts[j].indexOf("(")).trim());
						String temp2 = (parts[j].substring(
								parts[j].indexOf("(") + 1,
								parts[j].length() - 2).trim());
						if (temp2.equalsIgnoreCase("no"))
							keeper1 = "-" + temp1.substring(0, 1)
									+ temp1.substring(1);
						if (!temp2.equals("yes")) {
							keeper1 = temp2.substring(0, 1)
									+ temp2.substring(1) + ":"
									+ temp1.substring(0, 1)
									+ temp1.substring(1);
						} else {
							keeper1 = temp1.substring(0, 1)
									+ temp1.substring(1);
						}

					}
				}
				concepts.add(Object
						.getAntecedents()
						.get(i)
						.replaceAll("l_o_", "")
						.replaceAll(" and ", " AND ")
						.replaceAll("[0-9]", "")
						.replaceAll("[()]", "")
						.substring(
								0,
								Object.getAntecedents().get(i)
										.replaceAll("l_o_", "")
										.replaceAll(" and ", " AND ")
										.replaceAll("[0-9]", "")
										.replaceAll("[()]", "").length() - 1));
				associatedRules.add(new ProfileConcept(concepts, freq, tsTime,
						true, Object.getConfidence().get(i), null));
				keep.add(Object.getAntecedents().get(i).toString()
						.toLowerCase());

			}
		}

		return associatedRules;
	}

	/**
	 * Update function for concepts extracted from translateGAINFullExport.
	 * 
	 * @param userID
	 *            the user id
	 * @param simpleConceptsFromFile
	 *            the simple concepts
	 * @param simpleConceptsFromGainFullExport
	 *            the full export to prof concept list
	 * @return the list
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public static List<ProfileConcept> fullExportUpdate(String userID,
			List<ProfileConcept> simpleConceptsFromFile,
			List<ProfileConcept> simpleConceptsFromGainFullExport)
			throws IOException {

		List<ProfileConcept> fullExportUpdated = new ArrayList<ProfileConcept>();

		if (simpleConceptsFromFile.size() != 0) {
			for (int i = 0; i < simpleConceptsFromFile.size(); i++) {
				ArrayList<String> concepts = new ArrayList<String>();
				boolean choice = false;
				String concept = "";
				int frequency = simpleConceptsFromFile.get(i).getFreq();
				long tsTime = util.TimestampReader.getTimestamp();
				double weight = simpleConceptsFromFile.get(i).getWeight();
				innerloop: for (int k = 0; k < simpleConceptsFromGainFullExport
						.size(); k++) {
					if (simpleConceptsFromGainFullExport
							.get(k)
							.getConcept()
							.toLowerCase()
							.replaceAll("[^A-Za-z]+", "")
							.equals(simpleConceptsFromFile.get(i).getConcepts()
									.toString().toLowerCase()
									.replaceAll("[^A-Za-z]+", ""))) {
						concept = simpleConceptsFromFile.get(i).getConcepts()
								.toString().toLowerCase();
						frequency = simpleConceptsFromFile.get(i).getFreq()
								+ simpleConceptsFromGainFullExport.get(k)
										.getFreq();

						weight += simpleConceptsFromGainFullExport.get(k)
								.getWeight();
						choice = true;
						tsTime = simpleConceptsFromGainFullExport.get(k)
								.getLastTimestamp();
						break innerloop;
					} else {
						concept = simpleConceptsFromFile.get(i).getConcepts()
								.toString().toLowerCase();
						frequency = simpleConceptsFromFile.get(i).getFreq();
						weight = simpleConceptsFromFile.get(i).getWeight();
						tsTime = simpleConceptsFromFile.get(i)
								.getLastTimestamp();
						continue innerloop;
					}

				}
				concepts.add(concept.substring(1, concept.length() - 1));
				if (choice == true) {
					if (weight > 0) {

						fullExportUpdated.add(new ProfileConcept(concepts,
								frequency, tsTime, true, weight, null));
					} else {

						fullExportUpdated.add(new ProfileConcept(concepts,
								frequency, tsTime, false, weight, null));
					}
				} else {
					if (weight > 0) {

						fullExportUpdated.add(new ProfileConcept(concepts
								.toString().toLowerCase(), frequency, tsTime,
								true, weight, null));
					} else {

						fullExportUpdated.add(new ProfileConcept(concepts
								.toString().toLowerCase(), frequency, tsTime,
								false, weight, null));
					}
				}

			}
		} else {
			for (int i = 0; i < simpleConceptsFromGainFullExport.size(); i++) {

				ArrayList<String> concepts = new ArrayList<String>();
				String concept = simpleConceptsFromGainFullExport.get(i)
						.getConcept().toLowerCase();
				concepts.add(concept);
				long tsTime = simpleConceptsFromGainFullExport.get(i)
						.getLastTimestamp();
				int freq = simpleConceptsFromGainFullExport.get(i).getFreq();
				Double agWeight = simpleConceptsFromGainFullExport.get(i)
						.getWeight();
				if (agWeight > 0) {
					fullExportUpdated.add(new ProfileConcept(concepts, freq,
							tsTime, true, agWeight, null));
				} else {
					fullExportUpdated.add(new ProfileConcept(concepts, freq,
							tsTime, false, agWeight, null));
				}
			}

		}
		Collections.sort(fullExportUpdated, Collections.reverseOrder());

		return fullExportUpdated;
	}

	/**
	 * Update simple concepts.
	 * 
	 * @param fullExportUpdatedList
	 *            the full export updated list
	 * @return the list
	 */
	public static List<ProfileConcept> updateSimpleConcepts(
			List<ProfileConcept> fullExportUpdatedList) {
		ArrayList<ProfileConcept> concepts = new ArrayList<ProfileConcept>();
		boolean flag = false;
		Double weight = null;
		int freq = 0;
		for (int i = 0; i < fullExportUpdatedList.size(); i++) {
			weight = fullExportUpdatedList.get(i).getWeight();
			freq = fullExportUpdatedList.get(i).getFreq();
			double sum = 0.00;

			String concept = fullExportUpdatedList.get(i).getConcept();
			if (concept.equalsIgnoreCase("agent")) {
				System.out.println("ARXIKO WEIGHT " + weight);
			}
			for (int j = 0; j < fullExportUpdatedList.size(); j++) {
				if (i != j) {
					if (concept.equalsIgnoreCase(fullExportUpdatedList.get(j)
							.getConcept())) {
						sum += fullExportUpdatedList.get(j).getWeight();
						freq++;
						flag = true;
					}
				}
			}
			weight += sum;
			if (flag) {
				if (weight > 0) {
					concepts.add(new ProfileConcept(concept, freq,
							fullExportUpdatedList.get(i).getLastTimestamp(),
							true, weight, null, fullExportUpdatedList.get(i)
									.getGainInterest()));
					flag = false;
				} else {
					concepts.add(new ProfileConcept(concept, freq,
							fullExportUpdatedList.get(i).getLastTimestamp(),
							false, weight, null, fullExportUpdatedList.get(i)
									.getGainInterest()));
					flag = false;
				}
			} else {
				if (weight > 0) {
					concepts.add(new ProfileConcept(concept, freq,
							fullExportUpdatedList.get(i).getLastTimestamp(),
							true, weight, null, fullExportUpdatedList.get(i)
									.getGainInterest()));

				} else {
					concepts.add(new ProfileConcept(concept, freq,
							fullExportUpdatedList.get(i).getLastTimestamp(),
							false, weight, null, fullExportUpdatedList.get(i)
									.getGainInterest()));

				}
			}
			sum = 0.00;
		}
		for (int i = 0; i < concepts.size(); i++) {
			for (int j = 0; j < concepts.size(); j++) {
				if (i != j) {
					if (concepts
							.get(i)
							.getConcept()
							.replaceAll("[^A-Za-z]+", "")
							.equalsIgnoreCase(
									concepts.get(j).getConcept()
											.replaceAll("[^A-Za-z]+", ""))) {
						concepts.remove(i);
					}
				}
			}
		}
		for (int i = 0; i < concepts.size(); i++) {
			for (int j = 0; j < concepts.size(); j++) {
				if (i != j) {
					if (concepts
							.get(i)
							.getConcept()
							.replaceAll("[^A-Za-z]+", "")
							.equalsIgnoreCase(
									concepts.get(j).getConcept()
											.replaceAll("[^A-Za-z]+", ""))) {

						concepts.remove(j);
					}
				}
			}
		}
		return concepts;
	}

	/**
	 * Parse function that converts the concepts extracted from
	 * translateGAINFullExport to a List containing <br>
	 * ProfConceptLTV objects.
	 *
	 * @param userID            the userID
	 * @param accountID            the account id
	 * @param list the list
	 * @return the list containing the ProfileConcepts
	 * @throws IOException             Signals that an I/O exception has occurred.
	 */
	public static List<ProfileConcept> fullExportToProfConceptLTV(
			String userID, String accountID, List<GAINFullExport> list)
			throws IOException {
		List<Concept> cList = new ArrayList<Concept>();
		List<ProfileConcept> fullExportToProfConceptList = new ArrayList<ProfileConcept>();

		Double w = 0.00;
		long tsTime = 0;
		for (int i = 0; i < list.size(); i++) {
			tsTime = list.get(i).getSessionID();
			w = list.get(i).getInterest();
			Iterator<Map.Entry<String, Double>> it = list.get(i).getConcepts()
					.entrySet().iterator();
			while (it.hasNext()) {
				Map.Entry<String, Double> pairs = it.next();
				cList.add(new Concept(pairs.getKey(), pairs.getValue(), w,
						tsTime, list.get(i).getObjectID()));
			}

		}
		DecimalFormatSymbols europSymbols = new DecimalFormatSymbols(
				Locale.getDefault());
		europSymbols.setDecimalSeparator('.');
		DecimalFormat df = new DecimalFormat("#.#", europSymbols);
		df.setMaximumFractionDigits(2);
		for (int j = 0; j < cList.size(); j++) {
			if (cList.get(j).getGainInterest() != 0) {
				tsTime = cList.get(j).getTimestamp();
				int freq = 1;
				double agWeight = (cList.get(j).getWeight() * cList.get(j)
						.getGainInterest());
				System.out.println("weight apo FE " + cList.get(j).getWeight());
				System.out.println("interest apo FE "
						+ cList.get(j).getGainInterest());
				if (agWeight > 0) {
					fullExportToProfConceptList.add(new ProfileConcept(cList
							.get(j).getName().substring(0, 1).toUpperCase()
							+ cList.get(j).getName().substring(1), freq,
							tsTime, true, Double.parseDouble(df
									.format(agWeight)), null, cList.get(j)
									.getGainInterest()));
				} else {
					fullExportToProfConceptList.add(new ProfileConcept(cList
							.get(j).getName(), freq, tsTime, false, agWeight,
							null, cList.get(j).getGainInterest()));
				}

			}

		}
		return fullExportToProfConceptList;
	}

	/**
	 * Write user Profile to file.
	 * 
	 * @param total
	 *            the total
	 * @param outTotal
	 *            the out total
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public static void writeProfileToFile(List<ProfileConcept> total,
			BufferedWriter outTotal) throws IOException {
		for (int i = 0; i < total.size(); i++) {
			DecimalFormat df = new DecimalFormat("0.0000");
			if (total.get(i).getWeight() > 0) {

				if (total.get(i).getConcepts().toString().split(":")[0]
						.equals("[")) {

					outTotal.write(total.get(i).getLastTimestamp()
							+ "\t"
							+ total.get(i).getConcepts().toString()
									.replaceFirst(":", "") + "\t"
							+ df.format(total.get(i).getWeight()) + "\t"
							+ total.get(i).getFreq() + "\t" + true + "\t"
							+ null + "\n");

				} else {

					outTotal.write(total.get(i).getLastTimestamp() + "\t"
							+ total.get(i).getConcepts() + "\t"
							+ df.format(total.get(i).getWeight()) + "\t"
							+ total.get(i).getFreq() + "\t" + true + "\t"
							+ null + "\n");
				}
			} else {
				if (total.get(i).getInstance() != "") {
					outTotal.write(total.get(i).getFreq() + "\t"
							+ total.get(i).getLastTimestamp() + "\t"
							+ df.format(total.get(i).getWeight()) + "\t"
							+ false + "\t" + total.get(i).getInstance() + ":"
							+ total.get(i).getConcepts() + "\t" + null + "\n");

				} else {
					outTotal.write("\n" + total.get(i).getFreq() + "\t"
							+ total.get(i).getLastTimestamp() + "\t"
							+ df.format(total.get(i).getWeight()) + "\t"
							+ false + "\t" + total.get(i).getConcepts() + "\t"
							+ null + "\n");
				}
			}
		}
	}

	/**
	 * Full export gain entities annotation.
	 * 
	 * @param gainEntitiesExportList
	 *            the gain entities export list
	 * @param fullExportList
	 *            the full export list
	 * @param pclist
	 *            the pclist
	 * @return the list
	 */
	public static List<ProfileConcept> fullExportGainEntitiesAnnotation(
			List<data.GAINEntities> gainEntitiesExportList,
			List<data.GAINFullExport> fullExportList,
			List<ProfileConcept> pclist) {
		for (int i = 0; i < fullExportList.size(); i++) {
			int freq = 0;
			double agWeight;
			long tsTime = util.TimestampReader.getTimestamp();
			for (int j = 0; j < gainEntitiesExportList.size(); j++) {
				/** GAINFullExport - GAINEntities objectId Check */
				if (fullExportList
						.get(i)
						.getObjectID()
						.substring(
								1,
								fullExportList.get(i).getObjectID().toString()
										.length() - 1)
						.equals(gainEntitiesExportList.get(j).getObjectID())) {

					for (int k = 0; k < gainEntitiesExportList.get(j)
							.getEntities().size(); k++) {
						if (gainEntitiesExportList.get(j).getEntities().get(k)
								.getIType() != null
								&& gainEntitiesExportList.get(j).getEntities()
										.get(k).getEntityConfidence() != 0) {
							freq = 1;
							agWeight = gainEntitiesExportList.get(j)
									.getEntities().get(k).getEntityConfidence();

							pclist.add(new ProfileConcept(
									gainEntitiesExportList
											.get(j)
											.getEntities()
											.get(k)
											.getIType()
											.substring(
													gainEntitiesExportList
															.get(j)
															.getEntities()
															.get(k).getIType()
															.lastIndexOf("/") + 1),
									gainEntitiesExportList.get(j).getEntities()
											.get(k).getEntityLabel(), freq,
									tsTime, true, agWeight, null));

						}
					}
				}
			}
		}

		return pclist;
	}

	/**
	 * Update associacion rules.
	 *
	 * @param ts            the ts
	 * @param Object            the object
	 * @param associationRules            the association rules
	 * @param xml            the xml
	 * @param simpleConcepts the simple concepts
	 * @return the list
	 * @throws IOException             Signals that an I/O exception has occurred.
	 */
	public static List<ProfileConcept> updateAssociacionRules(
			PlainUserProfileCreator ts, PMMLParserHandler Object,
			List<ProfileConcept> associationRules, List<ProfileConcept> xml,
			List<ProfileConcept> simpleConcepts) throws IOException {

		if (associationRules.size() != 0) {

			for (int j = 0; j < ts.getAssociatedRules(Object).size(); j++) {
				int freq = 0;
				Long tsTime = null;
				Double weight = null;

				outerloop: for (int i = 0; i < associationRules.size(); i++) {

					if (ts.getAssociatedRules(Object)
							.get(j)
							.getConcept()
							.toLowerCase()
							.replaceAll("\\s+", "")
							.equals(associationRules.get(i).getConcept()
									.toLowerCase().replaceAll("\\s+", ""))) {
						freq = 1;
						weight = ts.getAssociatedRules(Object).get(j)
								.getWeight();
						tsTime = associationRules.get(i).getLastTimestamp();
						break outerloop;
					} else {
						freq = 1;
						weight = associationRules.get(i).getWeight();
						tsTime = ts.getAssociatedRules(Object).get(j)
								.getLastTimestamp();
						continue outerloop;
					}

				}
				if (weight > 0) {
					xml.add(new ProfileConcept(ts.getAssociatedRules(Object)
							.get(j).getConcept().toLowerCase()
							.replace(" and ", " AND "), freq, tsTime, true,
							weight, null));

				} else {
					xml.add(new ProfileConcept(ts.getAssociatedRules(Object)
							.get(j).getConcept().toLowerCase()
							.replace(" and ", " AND "), freq, tsTime, false,
							weight, null));

				}

			}
		} else {

			for (int i = 0; i < xml.size(); i++) {

				Long tsTime = null;
				ArrayList<String> concepts = new ArrayList<String>();
				concepts.add(xml
						.get(i)
						.getConcepts()
						.toString()
						.substring(
								1,
								xml.get(i).getConcepts().toString().length() - 1));
				xml.get(i).setConcepts(concepts);

				String concept0 = xml.get(i).getConcepts().toString()
						.split(" AND ")[0].substring(1);

				String concept1 = xml.get(i).getConcepts().toString()
						.split(" AND ")[1].substring(0,
						xml.get(i).getConcepts().toString().split(" AND ")[1]
								.length() - 1);

				for (int j = 0; j < simpleConcepts.size(); j++) {

					long max = simpleConcepts.get(j).getLastTimestamp();
					if (simpleConcepts.get(j).getConcepts().toString()
							.contains(concept0)
							|| simpleConcepts.get(j).getConcepts().toString()
									.contains(concept1)) {

						if (simpleConcepts.get(j).getLastTimestamp() < max) {
							xml.get(i).setLastTimestamp(
									simpleConcepts.get(j).getLastTimestamp());
						} else {
							xml.get(i).setLastTimestamp(max);
						}
					}
				}

				// if (weight > 0) {
				//
				// xml.add(new ProfileConcept(concepts, freq, tsTime, true,
				// weight, null));
				//
				// } else {
				// xml.add(new ProfileConcept(concepts, freq, tsTime, false,
				// weight, null));
				//
				// }

			}
		}
		return xml;
	}

	/**
	 * Update entities.
	 * 
	 * @param simpleConcepts
	 *            the simple concepts
	 * @param entitiesToBeUpdated
	 *            the entities concepts
	 * @param entitiesFromFile
	 *            the entities
	 * @param entitiesUpdatedList
	 *            the entities updated list
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public static void updateEntities(List<ProfileConcept> simpleConcepts,
			List<ProfileConcept> entitiesToBeUpdated,
			List<ProfileConcept> entitiesFromFile,
			List<ProfileConcept> entitiesUpdatedList) throws IOException {

		if (entitiesFromFile.size() != 0) {
			// Updating Profile Entities
			for (int j = 0; j < entitiesToBeUpdated.size(); j++) {
				ArrayList<String> concepts = new ArrayList<String>();
				String conceptFromFile = "";
				ProfileConcept temp = new ProfileConcept();

				int conceptToBeUpdatedFrequency = entitiesToBeUpdated.get(j)
						.getFreq();
				double conceptToBeUpdatedWeight = entitiesToBeUpdated.get(j)
						.getWeight();
				double weight = 0.00;
				long entityTimestamp = entitiesToBeUpdated.get(j)
						.getLastTimestamp();
				String conceptToBeUpdated = entitiesToBeUpdated
						.get(j)
						.getConcepts()
						.toString()
						.substring(
								1,
								entitiesToBeUpdated.get(j).getConcepts()
										.toString().length() - 1);

				for (int i = 0; i < entitiesFromFile.size(); i++) {
					weight = entitiesFromFile.get(i).getWeight();

					int freq = entitiesFromFile.get(i).getFreq();
					conceptFromFile = entitiesFromFile
							.get(i)
							.getConcepts()
							.toString()
							.substring(
									1,
									entitiesFromFile.get(i).getConcepts()
											.toString().length() - 1)
							.replace(" ", "");

					if (conceptFromFile
							.toLowerCase()
							.replaceAll("[^A-Za-z]+", "")
							.equalsIgnoreCase(
									conceptToBeUpdated.toLowerCase()
											.replaceAll("[^A-Za-z]+", ""))) {

						conceptToBeUpdatedWeight += weight;
						conceptToBeUpdatedFrequency += freq;
						temp.setFreq(conceptToBeUpdatedFrequency + freq);
						temp.setWeight(conceptToBeUpdatedWeight);
						temp.setLastTimestamp(entityTimestamp);

					}
				}
				concepts.add(entitiesToBeUpdated
						.get(j)
						.getConcepts()
						.toString()
						.substring(
								1,
								entitiesToBeUpdated.get(j).getConcepts()
										.toString().length() - 1));
				entitiesUpdatedList.add(new ProfileConcept(concepts,
						conceptToBeUpdatedFrequency, entityTimestamp, true,
						conceptToBeUpdatedWeight, null));

			}

			// Creation of Profile Entities Concepts
		} else {
			for (int i = 0; i < entitiesToBeUpdated.size(); i++) {

				try {
					Long tsTime = entitiesToBeUpdated.get(i).getLastTimestamp();
					ProfileConcept temp = new ProfileConcept();
					ArrayList<String> concepts = new ArrayList<String>();
					Integer freq = entitiesToBeUpdated.get(i).getFreq();
					Double weight = entitiesToBeUpdated.get(i).getWeight();
					String concept = entitiesToBeUpdated
							.get(i)
							.getConcepts()
							.toString()
							.toLowerCase()
							.substring(
									1,
									entitiesToBeUpdated.get(i).getConcepts()
											.toString().length() - 1);
					concepts.add(concept);

					for (int j = 0; j < simpleConcepts.size(); j++) {

						String entityConcept = entitiesToBeUpdated.get(i)
								.getConcepts().toString().toLowerCase()
								.split(":")[1].substring(0, entitiesToBeUpdated
								.get(i).getConcepts().toString().toLowerCase()
								.split(":")[1].length() - 1);
						String simpleConcept = simpleConcepts.get(j)
								.getConcepts().toString().toLowerCase();

						if (entityConcept.equals(simpleConcept)) {
							DecimalFormatSymbols europSymbols = new DecimalFormatSymbols(
									Locale.getDefault());
							europSymbols.setDecimalSeparator('.');
							DecimalFormat df = new DecimalFormat("#.#",
									europSymbols);
							df.setMaximumFractionDigits(2);
							temp.setLastTimestamp(simpleConcepts.get(j)
									.getLastTimestamp());
							temp.setFreq(freq);
							temp.setWeight(Double.parseDouble(df.format(weight)));
							temp.setConcept(concepts.toString());
							temp.setConcepts(concepts);
						}

					}
					DecimalFormatSymbols europSymbols = new DecimalFormatSymbols(
							Locale.getDefault());
					europSymbols.setDecimalSeparator('.');
					DecimalFormat df = new DecimalFormat("#.#", europSymbols);
					df.setMaximumFractionDigits(2);
					entitiesUpdatedList.add(new ProfileConcept(concepts, freq,
							tsTime, true,
							Double.parseDouble(df.format(weight)), null));

				} catch (Exception e) {

				}
			}
		}
	}

	/**
	 * Update user.
	 *
	 * @param plainUserProfileCreatorObject            the plain user profile creator object
	 * @param accountID            the account id
	 * @param userID            the user id
	 * @throws SAXException             the sAX exception
	 * @throws IOException             Signals that an I/O exception has occurred.
	 * @throws FileNotFoundException             the file not found exception
	 * @throws ParseException the parse exception
	 */
	public static void updateUser(
			PlainUserProfileCreator plainUserProfileCreatorObject,
			String accountID, String userID) throws SAXException, IOException,
			FileNotFoundException, ParseException {
		DecimalFormatSymbols europSymbols = new DecimalFormatSymbols(
				Locale.getDefault());
		europSymbols.setDecimalSeparator('.');
		DecimalFormat df = new DecimalFormat("0.00", europSymbols);

		// Call translateGAINLatestFullExport and organize results in
		// ProfileConcept
		// and GAINFullExport structures for later use in the process of
		// creating-
		// updating a user profile.

		List<GAINFullExport> gainFullExportConcepts = transform.transformLODtoLUMO
				.translateGAINLatestFullExportList(Config.FULL_EXPORT_ROOT
						+ accountID + Config.FULL_EXPORT_SUFFIX + userID);
		List<ProfileConcept> fullExportToProfConcept = fullExportToProfConceptLTV(
				userID, accountID, gainFullExportConcepts);
		// Read User Profile and create 3 different Lists<ProfileConcept>
		// containing simple concepts,
		// complex concepts and associated rules.
		List<ProfileConcept> simpleConceptsFromFile = new ArrayList<ProfileConcept>();
		List<ProfileConcept> complexConceptsFromFile = new ArrayList<ProfileConcept>();
		List<ProfileConcept> associatedRulesFromFile = new ArrayList<ProfileConcept>();
		readProfile(userID, simpleConceptsFromFile, complexConceptsFromFile,
				associatedRulesFromFile);

		// End of Read User Profile.

		/** ---Begin Simple Concepts Add/Update--- **/

		List<ProfileConcept> profile = new ArrayList<ProfileConcept>();
		List<ProfileConcept> simpleConcepts = fullExportUpdate(userID,
				simpleConceptsFromFile,
				updateSimpleConcepts(fullExportToProfConcept));
		List<ProfileConcept> entities = new ArrayList<ProfileConcept>();

		/** ---End Simple Concepts Add/Update--- **/

		/** ---Begin Complex Concepts Add/Update--- **/

		List<ProfileConcept> entitiesConcepts = new ArrayList<ProfileConcept>();
		List<ProfileConcept> entitiesToBeUpdated = new ArrayList<ProfileConcept>();

		fullExportToEntitiesAnnotation(accountID, entitiesConcepts,
				gainFullExportConcepts);
		entitiesUpdateDuplicates(entitiesConcepts, entitiesToBeUpdated, df);
		entitiesClearDuplicates(entitiesToBeUpdated);
		mineRulesURL(userID);
		readPMML(accountID, userID);
		updateEntities(simpleConcepts, entitiesToBeUpdated,
				complexConceptsFromFile, entities);
		entitiesClearDuplicates(entities);

		/** ---End Complex Concepts Add/Update--- **/

		/** ---Begin Association Rules Add/Update--- **/

		PMMLParserHandler pmmlParserHandlerObject = new PMMLParserHandler();
		XMLReader parser = XMLReaderFactory.createXMLReader();
		parser.setContentHandler(pmmlParserHandlerObject);
		parser.parse(Config.PMML);

		List<ProfileConcept> associatedRules = updateAssociacionRules(
				plainUserProfileCreatorObject, pmmlParserHandlerObject,
				associatedRulesFromFile,
				plainUserProfileCreatorObject
						.getAssociatedRules(pmmlParserHandlerObject),
				simpleConcepts);

		/** ---End Association Rules Add/Update--- **/

		profile.addAll(simpleConcepts);
		profile.addAll(entities);
		profile.addAll(associatedRules);

		Collections.sort(profile, Collections.reverseOrder());

		userProfileToFile(userID, profile);

	}

	/**
	 * Read user profile.
	 *
	 * @param userID            the user id
	 * @param simpleConceptsFromFile            the simple concepts from file
	 * @param complexConceptsFromFile            the complex concepts from file
	 * @param associatedRulesFromFile            the associated rules from file
	 * @throws FileNotFoundException             the file not found exception
	 * @throws ParseException the parse exception
	 */
	public static void readProfile(String userID,
			List<ProfileConcept> simpleConceptsFromFile,
			List<ProfileConcept> complexConceptsFromFile,
			List<ProfileConcept> associatedRulesFromFile)
			throws FileNotFoundException, ParseException {
		List<String> fileConcepts = util.FileOpenUtil
				.readFromFile(Config.PROFILE_ROOT + userID
						+ "_plain_profile.txt");
		NumberFormat nf = NumberFormat.getNumberInstance(Locale.getDefault());

		for (int l = 0; l < fileConcepts.size(); l++) {
			try {
				String[] splited = fileConcepts.get(l).split("\\s+");
				if (!splited[1].contains(":") && !splited[2].equals("AND")) {
					ArrayList<String> concepts = new ArrayList<String>();
					concepts.add(splited[1].toString().substring(1,
							splited[1].length() - 1));
					int freq = Integer.parseInt(splited[3]);
					long tsTime = Long.parseLong(splited[0]);
					double agWeight = nf.parse(splited[2]).doubleValue();
					boolean choice = Boolean.parseBoolean(splited[4]);
					simpleConceptsFromFile.add(new ProfileConcept(concepts,
							freq, tsTime, choice, agWeight, null));
				} else if (splited[1].contains(":")) {
					ArrayList<String> concepts = new ArrayList<String>();
					concepts.add(splited[1].substring(1,
							splited[1].length() - 1));
					complexConceptsFromFile.add(new ProfileConcept(concepts,
							Integer.parseInt(splited[3]), Long
									.parseLong(splited[0]), Boolean
									.parseBoolean(splited[4]), nf.parse(
									splited[2]).doubleValue(), null));
				} else {
					if (splited[4].contains("l_o")) {
						String str = fileConcepts.get(l);
						Pattern p = Pattern.compile(":");
						Matcher m = p.matcher(str);
						int count = 0;
						while (m.find()) {
							count++;
						}
						if (count == 3) {
							associatedRulesFromFile.add(new ProfileConcept(
									splited[4] + " " + splited[5] + " "
											+ splited[6] + " " + splited[7]
											+ " " + splited[8], Integer
											.parseInt(splited[0]), Long
											.parseLong(splited[1]), Boolean
											.parseBoolean(splited[3]), nf
											.parse(splited[2]).doubleValue(),
									null));
						} else if (count == 2) {
							associatedRulesFromFile.add(new ProfileConcept(
									splited[4] + " " + splited[5] + " "
											+ splited[6], Integer
											.parseInt(splited[0]), Long
											.parseLong(splited[1]), Boolean
											.parseBoolean(splited[3]), nf
											.parse(splited[2]).doubleValue(),
									null));
						} else if (count == 1) {
							associatedRulesFromFile.add(new ProfileConcept(
									splited[4], Integer.parseInt(splited[0]),
									Long.parseLong(splited[1]), Boolean
											.parseBoolean(splited[3]), nf
											.parse(splited[2]).doubleValue(),
									null));
						}
					}
				}

			} catch (NumberFormatException e) {
				e.printStackTrace();
			}

		}
	}

	/**
	 * Read pmml.
	 * 
	 * @param accountID
	 *            the account id
	 * @param userID
	 *            the user id
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws SAXException
	 *             the sAX exception
	 */
	public static void readPMML(String accountID, String userID)
			throws IOException, SAXException {
		try {
			readPMML.pmmlToFile(accountID, userID);
		} catch (TransformerConfigurationException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (TransformerFactoryConfigurationError e) {
			e.printStackTrace();
		} catch (TransformerException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Mine rules url.
	 * 
	 * @param userID
	 *            the user id
	 * @throws MalformedURLException
	 *             the malformed url exception
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public static void mineRulesURL(String userID)
			throws MalformedURLException, IOException {
		URL url = new URL(Config.API_ROOT + userID);
		InputStream is = url.openStream();
		try {
		} finally {
			is.close();
		}
	}

	/**
	 * User profile to file.
	 * 
	 * @param uid
	 *            the uid
	 * @param profile
	 *            the profile
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public static void userProfileToFile(String uid,
			List<ProfileConcept> profile) throws IOException {
		FileWriter totalStream = new FileWriter(Config.TEMP_FILE, true);
		BufferedWriter outTotalWriter = new BufferedWriter(totalStream);
		writeProfileToFile(profile, outTotalWriter);
		outTotalWriter.close();

		File original3 = new File(Config.PROFILE_ROOT + "/" + uid
				+ "_plain_profile.txt");
		original3.delete();
		File update = new File(Config.TEMP_FILE);
		update.renameTo(original3);
	}

	/**
	 * Full export to entities annotation.
	 * 
	 * @param accountID
	 *            the account id
	 * @param entities
	 *            the entities
	 * @param gainFullExportConcepts
	 *            the gain full export concepts
	 */
	public static void fullExportToEntitiesAnnotation(String accountID,
			List<ProfileConcept> entities,
			List<data.GAINFullExport> gainFullExportConcepts) {
		for (int i = 0; i < gainFullExportConcepts.size(); i++) {

			try {
				List<GAINEntities> gainEntities = transform.transformLODtoLUMO
						.translateGAINEntitiesList(Config.GAIN_ENTITIES_ROOT
								+ accountID
								+ Config.GAIN_ENTITIES_SUFFIX
								+ gainFullExportConcepts
										.get(i)
										.getObjectID()
										.substring(
												gainFullExportConcepts.get(i)
														.getObjectID()
														.lastIndexOf("/") + 1,
												gainFullExportConcepts.get(i)
														.getObjectID()
														.indexOf("?")));

				for (int j = 0; j < gainEntities.size(); j++) {

					if (gainFullExportConcepts
							.get(i)
							.getObjectID()
							.substring(
									gainFullExportConcepts.get(i).getObjectID()
											.lastIndexOf("/") + 1)
							.equals(gainEntities.get(j).getObjectID())
							&& gainFullExportConcepts.get(i).getInterest() != 0) {

						for (int k = 0; k < gainEntities.get(j).getEntities()
								.size(); k++) {

							if (gainEntities.get(j).getEntities().get(k)
									.getIType() != null
									&& gainEntities.get(j).getEntities().get(k)
											.getEntityConfidence() != 0) {
								DecimalFormatSymbols europSymbols = new DecimalFormatSymbols(
										Locale.getDefault());
								europSymbols.setDecimalSeparator('.');
								DecimalFormat df = new DecimalFormat("#.#",
										europSymbols);
								df.setMaximumFractionDigits(2);
								for (Entry<String, Double> entry : gainEntities
										.get(j).getEntities().get(k)
										.getNewconcs().entrySet()) {
									ArrayList<String> concepts = new ArrayList<String>();
									concepts.add(gainEntities.get(j)
											.getEntities().get(k)
											.getEntityLabel().toLowerCase()
											.replace(" ", "_")
											+ ":" + entry.getKey());
									String timestamp = gainEntities.get(j)
											.getID();
									//
									// Long tsTime = new BigInteger(timestamp,
									// 16)
									// .longValue();
									// long tsTime = new Date().getTime();
									// Long tsTime = Long.parseLong(timestamp);
									Long tsTime = gainFullExportConcepts.get(i)
											.getSessionID();
									int freq = 1;
									Double agWeight = gainEntities.get(j)
											.getEntities().get(k)
											.getEntityConfidence()
											* gainFullExportConcepts.get(i)
													.getInterest();
									entities.add(new ProfileConcept(concepts,
											freq, tsTime, true, Double
													.parseDouble(df
															.format(agWeight)),
											null));

								}

							}
						}
					}
				}
			} catch (Exception e) {
			}
		}
	}

	/**
	 * From unsigned hex.
	 *
	 * @param valueInUnsignedHex the value in unsigned hex
	 * @return the long
	 */
	public static final long fromUnsignedHex(final String valueInUnsignedHex) {
		long value = 0;

		final int hexLength = valueInUnsignedHex.length();
		if (hexLength == 0)
			throw new NumberFormatException("For input string: \"\"");
		for (int i = Math.max(0, hexLength - 16); i < hexLength; i++) {
			final char ch = valueInUnsignedHex.charAt(i);

			if (ch >= '0' && ch <= '9')
				value = (value << 4) | (ch - '0');
			else if (ch >= 'A' && ch <= 'F')
				value = (value << 4) | (ch - ('A' - 0xaL));
			else if (ch >= 'a' && ch <= 'f')
				value = (value << 4) | (ch - ('a' - 0xaL));
			else
				throw new NumberFormatException("For input string: \""
						+ valueInUnsignedHex + "\"");
		}

		return value;
	}

	/**
	 * Entities update duplicates.
	 * 
	 * @param entities
	 *            the entities
	 * @param entitiesToBeUpdated
	 *            the entities to be updated
	 * @param df
	 *            the df
	 */
	public static void entitiesUpdateDuplicates(List<ProfileConcept> entities,
			List<ProfileConcept> entitiesToBeUpdated, DecimalFormat df) {
		for (int i = 0; i < entities.size(); i++) {
			Double weight = entities.get(i).getWeight();
			int freq = entities.get(i).getFreq();
			for (int j = 0; j < entities.size(); j++) {
				if (i != j) {
					if (entities
							.get(i)
							.getConcepts()
							.toString()
							.toLowerCase()
							.equalsIgnoreCase(
									entities.get(j).getConcepts().toString()
											.toLowerCase())) {
						weight += entities.get(j).getWeight();
						freq++;

					}
				}
			}
			if (weight > 0) {
				entitiesToBeUpdated.add(new ProfileConcept(entities.get(i)
						.getConcepts(), freq, entities.get(i)
						.getLastTimestamp(), true, Double.parseDouble(df
						.format(weight)), null));
			} else {
				entitiesToBeUpdated.add(new ProfileConcept(entities.get(i)
						.getConcepts(), freq, entities.get(i)
						.getLastTimestamp(), false, weight, null));
			}
		}
	}

	/**
	 * Entities clear duplicates.
	 * 
	 * @param entitiesToBeUpdated
	 *            the entities to be updated
	 */
	public static void entitiesClearDuplicates(
			List<ProfileConcept> entitiesToBeUpdated) {
		try {
			for (int i = 0; i < entitiesToBeUpdated.size(); i++) {

				String temp1 = entitiesToBeUpdated.get(i).getConcepts()
						.toString().toLowerCase();
				for (int j = 0; j < entitiesToBeUpdated.size(); j++) {
					String temp2 = entitiesToBeUpdated.get(j).getConcepts()
							.toString().toLowerCase();
					if (i != j) {

						if (temp1.equalsIgnoreCase(temp2)) {

							entitiesToBeUpdated.remove(i);
						}
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Entities clear duplicates.
	 * 
	 * @param entities
	 *            the entities
	 * @param entitiesToBeUpdated
	 *            the entities to be updated
	 * @return the list
	 */
	public static List<ProfileConcept> entitiesClearDuplicates(
			List<ProfileConcept> entities,
			List<ProfileConcept> entitiesToBeUpdated) {

		for (int i = 0; i < entitiesToBeUpdated.size(); i++) {
			String instanceConcept = entitiesToBeUpdated.get(i).getInstance()
					+ ":" + entitiesToBeUpdated.get(i).getConcept();
			for (int j = 0; j < entitiesToBeUpdated.size(); j++) {
				String instanceConcept2 = entitiesToBeUpdated.get(j)
						.getInstance()
						+ ":"
						+ entitiesToBeUpdated.get(j).getConcept();
				if (i != j) {
					if (entities.get(i).getFreq() == entities.get(j).getFreq()
							&& instanceConcept
									.equalsIgnoreCase(instanceConcept2)) {

						entitiesToBeUpdated.remove(i);
					}
				}
			}
		}
		return entitiesToBeUpdated;
	}

	/**
	 * The main method. For testing purposes only.
	 * 
	 * @param args
	 *            the arguments
	 * @throws SAXException
	 *             the sAX exception
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public static void main(String[] args) throws SAXException, IOException {

//		long lStartTime = new Date().getTime();

		// PlainUserProfileCreator ts = new PlainUserProfileCreator();
		// String userID = "user20";
		// updateUser(ts, userID);

//		long lEndTime = new Date().getTime();
//
//		long difference = lEndTime - lStartTime;
//
//		System.out.println("\n" + "Elapsed milliseconds: " + difference);

	}

}
