/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>.
 */
package store;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import sys.Config;

/**
 * Contains I/O operations to access ontology-related data.
 * 
 * @author Dorothea Tsatsou
 * @author Pantelis Ieronimakis
 */
public class OntologyIO {

	/**
	 * Load the ontology.
	 * 
	 * @return the ontology
	 */
	@SuppressWarnings("resource")
	public static String loadTBox() {
		BufferedReader tboxReader = null;
		StringBuffer readBuffer = new StringBuffer();
		try {
			tboxReader = new BufferedReader(new FileReader(Config.TBOX_FILE));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String line;
		try {
			while ((line = tboxReader.readLine()) != null) {
				if (line.trim().length() > 0) {
					readBuffer.append(line);
					readBuffer.append('\n');
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		readBuffer.append('\n');
		return readBuffer.toString();
	}

	/**
	 * Load the mappings TBox.
	 *
	 * @return the mappings TBox
	 */
	public static String loadMappingsTBox() {
		BufferedReader tboxReader = null;
		StringBuffer readBuffer = new StringBuffer();
		FileReader fr = null;
		try {
			fr = new FileReader(Config.TBOX_MAPPINGS_FILE);
			tboxReader = new BufferedReader(fr);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}

		String line;
		try {
			while ((line = tboxReader.readLine()) != null) {
				if (line.trim().length() > 0) {
					readBuffer.append(line);
					readBuffer.append('\n');
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		readBuffer.append('\n');

		try {
			fr.close();
		} catch (IOException ee) {
			ee.printStackTrace();
		}
		// System.out.println(readBuffer.toString());
		return readBuffer.toString();
	}

	/**
	 * Load a subset of the mappings TBox based on specific concepts.
	 *
	 * @param concepts the concepts
	 * @return the mappings TBox
	 */
	public static String loadMappingsSubsetTBox(Set<String> concepts) {
		BufferedReader tboxReader = null;
		StringBuffer readBuffer = new StringBuffer();
		FileReader fr = null;
		try {
			fr = new FileReader(Config.TBOX_MAPPINGS_FILE);
			tboxReader = new BufferedReader(fr);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}

		String line;
		try {
			while ((line = tboxReader.readLine()) != null) {

				if (line.trim().length() > 0) {
					for (Iterator<String> iter = concepts.iterator(); iter
							.hasNext();) {
						if (line.trim()
								.replaceAll("[^A-Za-z]+", "")
								.toLowerCase()
								.contains(
										iter.next()
												.replaceAll("[^A-Za-z]+", "")
												.toLowerCase())) {
							readBuffer.append(line);
							readBuffer.append('\n');
						}
					}
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		readBuffer.append('\n');
		try {
			fr.close();
		} catch (IOException ee) {
			ee.printStackTrace();
		}
		return readBuffer.toString();
	}

	/**
	 * Load TBox concepts.
	 *
	 * @return the concepts
	 */
	@SuppressWarnings("resource")
	public static Vector<String> loadTBoxConcepts() {
		BufferedReader tboxReader = null;
		StringBuffer readBuffer = new StringBuffer();
		Vector<String> concepts = new Vector<String>();
		try {
			tboxReader = new BufferedReader(new FileReader(
					"resources/lumo_mappings.txt"));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String line;
		try {
			while ((line = tboxReader.readLine()) != null) {
				if (line.trim().length() > 0) {
					readBuffer.append(line);
					readBuffer.append('\n');
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		readBuffer.append('\n');
		String[] lines = readBuffer.toString().split("\n");
		for (int i = 0; i < lines.length; i++) {
			concepts.add(lines[i]);
		}
		return concepts;
	}

}