/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>.
 */
package store;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

import util.StringEditUtil;
import data.Pointer;
import data.Rule;

/**
 * The Class PMMLParserHandler. Parses the association rules retrieved from the 
 * GAIN preference learning module.
 * 
 * @author Pantelis Ieronimakis
 */
public class PMMLParserHandler extends DefaultHandler {

	/** The decision. */
	private boolean decision = false;
	
	/** The rules number. */
	private int rulesNumber = 0;
	
	/** The acc. */
	private StringBuilder acc;

	/** The Results. */
	private List<String> Results = Collections
			.synchronizedList(new ArrayList<String>());
	
	/** The Rules. */
	private List<String> Rules = Collections
			.synchronizedList(new ArrayList<String>());
	
	/** The Antecedents. */
	private List<String> Antecedents = Collections
			.synchronizedList(new ArrayList<String>());
	
	/** The Contecedents. */
	private List<String> Contecedents = Collections
			.synchronizedList(new ArrayList<String>());
	
	/** The Confidence. */
	private List<Double> Confidence = Collections
			.synchronizedList(new ArrayList<Double>());
	
	/** The Support. */
	private List<Double> Support = Collections
			.synchronizedList(new ArrayList<Double>());
	
	/** The Pointers. */
	private List<Pointer> Pointers = Collections
			.synchronizedList(new ArrayList<Pointer>());

	/** The rule list. */
	private List<Rule> ruleList = Collections
			.synchronizedList(new ArrayList<Rule>());
	
	/** The concept value list. */
	private List<Rule.ConceptValue> conceptValueList = Collections
			.synchronizedList(new ArrayList<Rule.ConceptValue>());

	/** The Confmap. */
	private Map<String, Double> Confmap = new ConcurrentHashMap<String, Double>(
			16, (float) 0.9, 1);
	
	/** The Suppmap. */
	private Map<String, Double> Suppmap = new ConcurrentHashMap<String, Double>(
			16, (float) 0.9, 1);

	/**
	 * Constructs a new PMMLParserHandler instance.
	 */

	public PMMLParserHandler() {
		acc = new StringBuilder();
	}

	/* (non-Javadoc)
	 * @see org.xml.sax.helpers.DefaultHandler#startDocument()
	 */
	@Override
	public void startDocument() {
		String startDocumentString = "Parsing started -----  Discovered Rules :";
		// System.out.println(startDocumentString);
	}

	/* (non-Javadoc)
	 * @see org.xml.sax.helpers.DefaultHandler#endDocument()
	 */
	@Override
	public void endDocument() {
		String endDocumentString = "\n \nParsing ended ------";
		/** Remove empty rows from list. */
		Results.removeAll(Arrays.asList("", null));
		/** Convert ArrayList to Array and Array Print. */
		String[] Result = Results.toArray(new String[Results.size()]);

		/** AssociativeRules Print. */
		for (int i = 0; i < rulesNumber; i++) {
			String temp = Result[(0 + (3 * i))].replaceAll("&", "AND");
			setRules(temp);
			setPointer(temp);
			String[] tempor = Result[(0 + (3 * i))].split(">:<");
			setContecedents(tempor[1]);
			String result1 = tempor[0].replaceAll("&", "AND");

			setAntecedents(result1.replace("Yes", ""));
			String result2 = tempor[1].replaceAll("&", "AND");

			/** Confidence Print */
			Double d = Double.parseDouble(Result[(1 + (3 * i))]);
			Confidence.add(d);
			Double tmp = Double.parseDouble(Result[(1 + (3 * i))]);
			Confmap.put(Result[(0 + (3 * i))], tmp);

			/** Support Print */
			Double dd = Double.parseDouble(Result[(2 + (3 * i))]);
			Support.add(dd);
			Double tmp2 = Double.parseDouble(Result[(2 + (3 * i))]);
			Suppmap.put(Result[(0 + (3 * i))], tmp2);

			StringEditUtil se = new StringEditUtil();

			Rule r = new Rule(temp, result1, tempor[1], d, dd);
			ruleList.add(r);

			se.conceptParser(result1);
			se.valueParser(result1);

			for (int k = 0; k < se.conceptParser(result1).size(); k++) {
				Rule rr = new Rule();
				Rule.ConceptValue innerObject = rr.new ConceptValue(se
						.conceptParser(result1).get(k), se.valueParser(result1)
						.get(k));
				conceptValueList.add(innerObject);
			}

			se.conceptParser(tempor[1].trim());
			se.valueParser(tempor[1].trim());

			for (int j = 0; j < se.conceptParser(tempor[1].trim()).size(); j++) {
				Rule rr = new Rule();
				Rule.ConceptValue innerObject = rr.new ConceptValue(se
						.conceptParser(tempor[1].trim()).get(j), se
						.valueParser(tempor[1].trim()).get(j));
				conceptValueList.add(innerObject);
			}

		}

	}

	/* (non-Javadoc)
	 * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
	 */
	@Override
	public void startElement(String nameSpaceURI, String localName,
			String qName, Attributes atts) {

		/** AssociationRule Check */
		if (qName.equals("AssociationRule")) {
			setRulesNumber();
			setDecision();
			String arString = "\nAssociative Rule: ";
		}

		/** Confidence Check */
		for (int i = 0; i < atts.getLength(); i++) {
			if (atts.getValue(i).equals("Conf")) {
				String confString = "\nConfidence: ";
				setDecision();
			}

			/** Support Check */
			if (atts.getValue(i).equals("Supp")) {
				String suppString = "\nSupport: ";
				setDecision();
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.xml.sax.helpers.DefaultHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override
	public void endElement(String nameSpaceURI, String localName, String qName) {
		String temp = (acc.toString()).trim();
		Results.add(temp);
		acc.setLength(0);
		unsetDecision();
	}

	/* (non-Javadoc)
	 * @see org.xml.sax.helpers.DefaultHandler#characters(char[], int, int)
	 */
	@Override
	public void characters(char[] ch, int start, int length) {
		if (decision)
			acc.append(ch, start, length);
	}

	/**
	 * Sets boolean decision to true so that specific line of the .xml file is
	 * parsed.
	 */

	private void setDecision() {
		decision = true;
	}

	/**
	 * Unsets boolean decision to true so that specific line of the .xml file is
	 * parsed.
	 */

	private void unsetDecision() {
		decision = false;
	}

	/**
	 * Keep Track of Rules Number.
	 */

	private synchronized void setRulesNumber() {
		rulesNumber += 1;
	}

	/**
	 * Sets the following Rule to the Rules ArrayList.
	 *
	 * @param s the new rules
	 */

	public void setRules(String s) {
		Rules.add(s);
	}

	/**
	 * Gets the ArrayList containing the Assosiation Rules discovered from
	 * the GAIN preference learning module.
	 * 
	 * @return the list of Rules.
	 */

	public List<String> getRules() {
		return Rules;
	}

	/**
	 * Gets the ArrayList that contains the Antecedents derived from the Pmml rules
	 * file.
	 *
	 * @param s the new antecedents
	 * @return the ArrayList.
	 */

	public void setAntecedents(String s) {
		Antecedents.add(s);
	}

	/**
	 * Sets the following Contecedent to the Contecendents ArrayList.
	 *
	 * @param s the new contecedents
	 */

	public void setContecedents(String s) {
		Contecedents.add(s);
	}

	/**
	 * Gets the ArrayList that contains the Antecedents derived from the Pmml
	 * file.
	 * 
	 * @return the ArrayList.
	 */

	public List<String> getAntecedents() {
		return this.Antecedents;
	}

	/**
	 * Gets the ArrayList that contains the Contecedents derived from the Pmml
	 * file.
	 * 
	 * @return the ArrayList.
	 */

	public List<String> getContecedents() {
		return Contecedents;
	}

	/**
	 * ArrayList<String> Print Method.
	 *
	 * @param alist the alist
	 * @param listsize the listsize
	 */

	public void printList(List<String> alist, int listsize) {
		for (int i = 0; i < listsize; i++) {
			System.out.println(alist.get(i));
		}
	}

	/**
	 * ArrayList<Pointer_Object> Print Method.
	 *
	 * @param list the list
	 * @param listsize the listsize
	 */

	public void printListObj(List<Pointer> list, int listsize) {
		for (int i = 0; i < listsize; i++) {
			System.out.println(list.get(i).getPointerValue(list.get(i)));
		}
	}

	/**
	 * Create a new Pointer Object and add it to the ArrayList that contains
	 * each one of them.
	 *
	 * @param s the new pointer
	 */

	public void setPointer(String s) {
		Pointer pnt = new Pointer(s);
		Pointers.add(pnt);
	}

	/**
	 * Gets the ArrayList containing pointers to Antecedent part of discovered
	 * rules from Easy-Miner.
	 * 
	 * @return the ArrayList.
	 */

	public List<Pointer> getPointers() {
		return Pointers;
	}

	/**
	 * Sets the following Confidence value to the Confidence ArrayList.
	 *
	 * @param d the new confidence
	 */

	public void setConfidence(Double d) {
		Confidence.add(d);
	}

	/**
	 * Gets the ArrayList containing Confidence values per Rule.
	 * 
	 * @return the ArrayList.
	 */

	public List<Double> getConfidence() {
		return Confidence;
	}

	/**
	 * Gets the ConcurrentHashMap that contains Rules-Confidence value derived
	 * from the Pmml file.
	 * 
	 * @return the ConcurrentHashMap.
	 */

	public Map<String, Double> getConfMap() {
		return Confmap;
	}

	/**
	 * Sets the following Support value to the Support ArrayList.
	 *
	 * @param d the new support
	 */

	public void setSupport(Double d) {
		Support.add(d);
	}

	/**
	 * Gets the ArrayList containing Support values per Rule.
	 * 
	 * @return the ArrayList.
	 */

	public List<Double> getSupport() {
		return Support;
	}

	/**
	 * Gets the ConcurrentHashMap that contains Rules-Support value derived from
	 * the Pmml file.
	 * 
	 * @return the ConcurrentHashMap.
	 */

	public Map<String, Double> getSuppMap() {
		return Suppmap;
	}

	/**
	 * Method that prints all details derived from parsex PMML file.
	 *
	 * @param myHandlerObject the my handler object
	 */

	public void printAll(PMMLParserHandler myHandlerObject) {
		System.out.println("Rules : ");
		System.out.println(myHandlerObject.getRules() + "\n");
		System.out.println("Antecedents : ");
		System.out.println(myHandlerObject.getAntecedents() + "\n");
		System.out.println("Contedecents : ");
		System.out.println(myHandlerObject.getContecedents() + "\n");
		System.out.println("Confidence Values : ");
		System.out.println(myHandlerObject.getConfidence() + "\n");
		System.out.println("Support Values : ");
		System.out.println(myHandlerObject.getSupport() + "\n");

	}

	/**
	 * Method that prints hash-key pairs from a given HashMap.
	 *
	 * @param map the map
	 */

	public void printHashMap(Map<String, Double> map) {
		Iterator<String> keySetIterator = map.keySet().iterator();
		while (keySetIterator.hasNext()) {
			String key = keySetIterator.next();
			System.out.println("key: \t" + key + " value: \t" + map.get(key));
		}
	}

	/**
	 * Method that prints hash-key pairs from a given Rule-Confidence HashMap.
	 *
	 * @param map the map
	 */

	public void printConfMap(Map<String, Double> map) {
		Iterator<String> keySetIterator = map.keySet().iterator();
		while (keySetIterator.hasNext()) {
			String key = keySetIterator.next();
			String temp = key.replaceAll("&", "AND");
			System.out.println("Rule: " + temp + "\t" + "Confidence value: \t"
					+ map.get(key));
		}
	}

	/**
	 * Method that prints hash-key pairs from a given Rule-Suppor HashMap.
	 *
	 * @param map the map
	 */

	public void printSuppMap(Map<String, Double> map) {
		Iterator<String> keySetIterator = map.keySet().iterator();
		while (keySetIterator.hasNext()) {
			String key = keySetIterator.next();
			String temp = key.replaceAll("&", "AND");
			System.out.println("Rule: \t" + temp + "\t" + "Support value: \t"
					+ map.get(key));
		}
	}

	/**
	 * Method that converts an Arraylist to a ConcurrentHashMap.
	 *
	 * @param alist the alist
	 * @param alsize the alsize
	 * @return the map
	 */

	public Map<String, Double> anteToHash(List<String> alist, Integer alsize) {
		StringEditUtil stringEditObject = new StringEditUtil();
		Map<String, Double> hashMap = new ConcurrentHashMap<String, Double>(16,
				(float) 0.9, 1);
		for (int i = 0; i < alsize; i++) {
			String Antecedent = alist.get(i);
			stringEditObject.stringParser(Antecedent);
			hashMap.putAll(stringEditObject.stringParser(Antecedent));
			printHashMap(stringEditObject.stringParser(Antecedent));
		}
		return hashMap;
	}

	/**
	 * Gets the List containing the Rule-Objects created.
	 * 
	 * @return the List.
	 */

	public List<Rule> getRule() {
		return this.ruleList;
	}

	/**
	 * Prints the associative rules which pointers reference.
	 *
	 * @param mHObject the m h object
	 */

	public void usePointers(PMMLParserHandler mHObject) {
		for (int i = 0; i < mHObject.getPointers().size(); i++) {
			System.out.println(mHObject.getPointers().get(i)
					.getPointerValue((mHObject.getPointers().get(i))));
		}
	}

	/**
	 * Gets the concept value rule.
	 *
	 * @return the concept value rule
	 */
	public List<Rule.ConceptValue> getConceptValueRule() {
		return this.conceptValueList;
	}

}
