/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>.
 */
package store;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import sys.Config;

/**
 * Contains I/O operations to access and manage profile-related data.
 * 
 * @author Pantelis Ieronimakis
 * @author Dorothea Tsatsou
 */
public class ProfileIO {

	/**
	 * Read plain profile from file.
	 *
	 * @param userID the user id
	 * @return the string
	 * @throws FileNotFoundException the file not found exception
	 */
	public static String readPlainProfileFromFile(String userID)
			throws FileNotFoundException {
		BufferedReader profileReader = null;
		StringBuffer readBuffer = new StringBuffer();
		String line;
		try {
			File file = new File(Config.PROFILE_ROOT + "/" + userID + ".txt");
			profileReader = new BufferedReader(new FileReader(file));
			while ((line = profileReader.readLine()) != null) {
				if (line.trim().length() > 0) {
					readBuffer.append(line);
					readBuffer.append('\n');
				}
			}
			profileReader.close();
		} catch (IOException e) {

		}

		readBuffer.append('\n');
		return readBuffer.toString();
	}
	
	/**
	 * Load the user profile for a given user.
	 * 
	 * @param userID
	 *            the user id
	 * 
	 * @return the user profile
	 */
	@SuppressWarnings("resource")
	public static String loadUserProfile(String userID) {
		BufferedReader tboxReader = null;
		StringBuffer readBuffer = new StringBuffer();
		try {
			tboxReader = new BufferedReader(new FileReader(
					Config.getUserProfileFile(userID)));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String line;
		try {
			while ((line = tboxReader.readLine()) != null) {
				if (line.trim().length() > 0) {
					readBuffer.append(line);
					readBuffer.append('\n');
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		readBuffer.append('\n');
		return readBuffer.toString();
	}

	/**
	 * Creates a file containing the user profile rule. If a file with the
	 * specified name already exists in the path, the system overwrites it. <br>
	 * The name of the file is the username.
	 * 
	 * @param newFileName
	 *            - the name of the file to be created/ovewritten.
	 * @param user
	 *            the user
	 */
	public static void createUserProfileFile(String newFileName, String user) {
		String userProfile = user;
		String[] userProfileLines = userProfile.split("\n");
		BufferedWriter profileWriter = null;
		try {
			File file = new File(Config.KRSS_USER_PROFILE_ROOT + newFileName
					+ ".txt");
			profileWriter = new BufferedWriter(new FileWriter(file));

			for (int i = 0; i < userProfileLines.length; i++) {
				profileWriter.write(userProfileLines[i], 0,
						userProfileLines[i].length());
				profileWriter.newLine();
			}
			profileWriter.close();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				profileWriter.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
}
