/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>.
 */
package sys;

/**
 * Configuration class intended to provide global reference to input and output paths and URLs.
 * 
 * @author Pantelis Ieronimakis
 * @author Dorothea Tsatsou
 */
public class Config {

	/** The root directory containing the ontology knowledge base files. */
	private static final String TBOX_REPOSITORY_ROOT = "resources/tbox/";

	/** The text file containing the whole ontology (terminological box) file. */
	public static final String TBOX_FILE = TBOX_REPOSITORY_ROOT + "lumo.txt";

	/** The text file containing lumo mappings. */
	public static final String TBOX_MAPPINGS_FILE = TBOX_REPOSITORY_ROOT
			+ "lumo_mappings.txt";

	/** The text file containing lumo concepts. */
	public static final String TBOX_CONCEPTS_FILE = TBOX_REPOSITORY_ROOT
			+ "lumo_concepts.txt";

	/** The root directory containing the user profile files. */
	public static final String KRSS_USER_PROFILE_ROOT = "resources/KRSS user profiles/";

	/**
	 * Gets the user profile file name for the given user id.
	 * 
	 * @param userID
	 *            the user id
	 * 
	 * @return the user profile filepath
	 */
	public static String getUserProfileFile(String userID) {
		return KRSS_USER_PROFILE_ROOT + userID + ".txt";
	}
	
	/** The Constant RESOURCES_ROOT. */
	public static final String RESOURCES_ROOT = "resources/";

	/** The Constant API_ROOT. */
	public static final String API_ROOT = "http://1.2.3.4/api/mine_rules?uid=";

	/** The Constant PMML. */
	public static final String PMML = "myOutput.xml";

	/**  PMML url *. */
	public static final String PMML_ROOT = "http://dev.wa.vse.cz/pl/api/";

	/** The Constant PMML_SUFFIX. */
	public static final String PMML_SUFFIX = "/rules?uid=";

	/**  FullExport url *. */
	public static final String FULL_EXPORT_ROOT = "http://dev.wa.vse.cz/gain/api/";

	/** The Constant FULL_EXPORT_SUFFIX. */
	public static final String FULL_EXPORT_SUFFIX = "/export/interests?uid=";
	
	/** The Constant GET_SESSIONS_NUMBER_SUFFIX. */
	public static final String GET_SESSIONS_NUMBER_SUFFIX = "/session/number?uid=";

	/**  GAINEntities root url *. */
	public static final String GAIN_ENTITIES_ROOT = "http://dev.wa.vse.cz/gain/api/";

	/** The Constant GAIN_ENTITIES_SUFFIX. */
	public static final String GAIN_ENTITIES_SUFFIX = "/object/attributes?pid=";

	/** The root directory containing the user profile files. */
	public static final String PROFILE_ROOT = "resources/user_profiles/";

	/** The root directory containing the XML file. */
	public static final String PMML_FILE = "resources/XMLFile.xml";

	/** The root directory containing the gainEntities updated file. */
	public static final String ENTITIES_FILE = "resources/entitiesFIle";

	/** The root directory containing the XML updated file. */
	public static final String PMML_UPDATED_FILE = "resources/PMMLFile";

	/** The root directory containing the fullExport updated file. */
	public static final String FULL_EXPORT_UPDATED_FILE = "resources/simpleConceptsFile";

	/** The Constant FULL_EXPORT_UPDATED_ROOT6. */
	public static final String FULL_EXPORT_UPDATED_ROOT6 = "resources/simpleConceptsUpdated";

	/** The root directory containing a temporary file used for update. */
	public static final String TEMP_FILE = "resources/temp.txt/";
	
	/** The Constant PMML_UPDATED. */
	public static final String PMML_UPDATED = "resources/pmmlFile.txt/";
	
	/** The Constant TEMP_ROOT. */
	public static final String TEMP_ROOT = "resources/temp.txt/";
	/** The path and file name of the passwords file. */
	public static final String PASSWORDS_FILE_PATH = "resources/passwords.txt";

	/** The root directory containing the user log files. */
	public static final String USER_LOGS_FILE_PATH = "resources/userLogs/";

	/**
	 * The maximum number of profiles that should be kept in the file. When this
	 * limit is exceeded profiles get pruned.
	 */
//	public static final int MAX_PROFILE_CONCEPTS_LIMIT = 10000;

	/**
	 * The amount that the minimum weight should be increased every time the
	 * concepts are to be pruned.
	 */
	public static final double PRUNING_STEP = 0.05;

	/**
	 * The time in milliseconds that must pass from the last update in order for
	 * the specific user's profile files to be considered outdated.
	 */
	public static final Long UPDATE_INTERVAL = 14400000l;

	/**
	 * When the number of milliseconds from the last transaction exceeds the
	 * number described her ethe server is considered idle.
	 */
	public static final Long IDLE_TIME = 1800000l;

}