/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>.
 */
package test;

import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.util.Date;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactoryConfigurationError;

import org.xml.sax.SAXException;

import profile.PlainUserProfileCreator;
import sys.Config;

/**
 * Class used to create-update a user profile based upon
 * translateGAINFullExport,translateGAINEntities and PMML parsing.
 * 
 * @author Pantelis Ieronimakis
 */
public class TestPlainUserProfileCreator {

	/**
	 * Initialization function for all files of the given user.
	 * 
	 * @param userID
	 *            the user id
	 */
	public void userInit(String userID) {

		try {
			String[] filepath = { Config.PROFILE_ROOT + userID
					+ "_plain_profile.txt" };

			for (int i = 0; i < filepath.length; i++) {
				File file = new File(filepath[i]);
				if (!file.exists()) {
					file.createNewFile();
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * The main method.
	 *
	 * @param args            the arguments
	 * @throws SAXException             the sAX exception
	 * @throws IOException             Signals that an I/O exception has occurred.
	 * @throws TransformerConfigurationException             the transformer configuration exception
	 * @throws ParserConfigurationException             the parser configuration exception
	 * @throws TransformerFactoryConfigurationError             the transformer factory configuration error
	 * @throws TransformerException             the transformer exception
	 * @throws ParseException the parse exception
	 */
	public static void main(String[] args) throws SAXException, IOException,
			TransformerConfigurationException, ParserConfigurationException,
			TransformerFactoryConfigurationError, TransformerException,
			ParseException {

		long lStartTime = new Date().getTime();

		PlainUserProfileCreator ts = new PlainUserProfileCreator();
		TestPlainUserProfileCreator ts2 = new TestPlainUserProfileCreator();
		String uid = "7";
		String accountID = "YOUTUBE-TEST";

		ts2.userInit(uid);
		profile.PlainUserProfileCreator.updateUser(ts, accountID, uid);

		long lEndTime = new Date().getTime();

		long difference = lEndTime - lStartTime;

		System.out.println("\n" + "Elapsed milliseconds: " + difference);

	}

}
